package GLite::HTTPS;

require 5.006;

use strict;
use FileHandle;

our $VERSION = 1.1;

=pod

=head1 NAME

EDG::HTTPS

=head1 SYNOPSIS

Call B<GLite::HTTPS::setEnv();> to use authenticated HTTPS.

=head1 DESCRIPTION

The main function of this package is B<setEnv()>, which sets the environmental
variables for the Crypt::SSLeay package, so it will able to authenticate to
a remote service, e.g. through HTTPS. 

If the effective userid is 0 (root), then the host key and certificate is 
configured.

If the effective userid is not 0 (non-root), then the proxy certificate is
configured: it is split up to three files and set as I<certificate>, 
I<key> and <CA>. If the proxy is not found, or there was a problem splitting
the file, then the user's real certificate is configured. In the later case
the SSL library will ask for a decryption password on the command line.

The split proxy will be cleaned up, when the program finishes. The cleanup
routine can also be called directly by B<cleanSplitProxy()>.

This module also tries to configure the L<IO::Socket::SSL> module
properly, if it is used. If this module is not loaded, then it
fails without any problem.

=head1 OPTIONS

The behaviour can be altered through environmental variables:

=over 4

=item X509_USER_PROXY

The location of the proxy certificate (and key). The default location is
C</tmp/x509up_p#uid#>

=item X509_USER_CERT

The location of the user certificate. Only used, if proxy is not found.

The default location for users: C<$HOME/.globus/usercert.pem>

The default location for host/service: C</etc/grid-security/hostcert.pem>

=item X509_USER_KEY

The location of the user key. Only used, if proxy is not found.

The default location for users: C<$HOME/.globus/userkey.pem>

The default location for host/service: C</etc/grid-security/hostkey.pem>

=item X509_CERTDIR

The location of the certificate directory. The default location is
C</etc/grid-security/certificates>

=back

=head1 AUTHORS

Akos Frohner <Akos.Frohner@cern.ch>, Karoly Lorentey <Lorentey.Karoly@elte.hu>

Copyright (c) 2003-2004 CERN, ELTE on behalf of the EU DataGrid and EGEE projects.
For license conditions see LICENSE file or
L<http://www.edg.org/license.html>.

=cut

################################################################################
# non-proxy settings
###

my $proxy; # location of the proxy certificate
my $debug = 0;

sub setEnv {
    require Crypt::SSLeay; # to make sure it exists

	# euid is not 0: this is a user with user cert or proxy
	if($>) {
		$proxy = $ENV{X509_USER_PROXY} || "/tmp/x509up_u" . $>;

		eval { splitProxy(); };

		if($@) {
			warn "Couldn't find the proxy file at $proxy\n$@";
		
			# fallback solution: the real user certificate
			$ENV{HTTPS_CERT_FILE} = $ENV{X509_USER_CERT} ||
				$ENV{HOME}.'/.globus/usercert.pem';
			$ENV{HTTPS_KEY_FILE}  = $ENV{X509_USER_KEY} ||
				$ENV{HOME}.'/.globus/userkey.pem';
		}
	}
	# euid is 0: this is a daemon, so we use the hostcert/key
	else {
		$ENV{HTTPS_CERT_FILE} = $ENV{X509_USER_CERT} ||
		  '/etc/grid-security/hostcert.pem';
		$ENV{HTTPS_KEY_FILE}  = $ENV{X509_USER_KEY} ||
		  '/etc/grid-security/hostkey.pem';
	}

    die "Certificate does not exists: $ENV{HTTPS_CERT_FILE}\n"
        unless -f $ENV{HTTPS_CERT_FILE};
    die "Private key does not exists: $ENV{HTTPS_KEY_FILE}\n"
        unless -f $ENV{HTTPS_KEY_FILE};

    # at this point we have certificate and key, so we can 
    # do the compatibility step for IO::Socket::SSL
    if ( exists($INC{'IO/Socket/SSL.pm'}) ) {
        #warn "Configuring IO::Socket::SSL with certificates\n";
        $IO::Socket::SSL::GLOBAL_CONTEXT_ARGS->{SSL_cert_file}=$ENV{HTTPS_CERT_FILE};
        $IO::Socket::SSL::GLOBAL_CONTEXT_ARGS->{SSL_key_file}=$ENV{HTTPS_KEY_FILE};
        $IO::Socket::SSL::GLOBAL_CONTEXT_ARGS->{SSL_use_cert}=1;
    }
    
	$ENV{HTTPS_VERSION} = 3;
	$ENV{HTTPS_CA_DIR} = $ENV{X509_CERTDIR} || 
		'/etc/grid-security/certificates';
}

################################################################################
# proxy handling for HTTPS
###

my $isSplit= 0; # the proxy file is split

# Splits up the proxy in three pieces, so we can 
# specify the individual pieces for Crypt:SSLeay.
#
# Unfortunately the SSLeay library cannot find 
# the individual pieces, if it is pointed to the
# same file, both for the certificate and the key.
sub splitProxy {
	die "Proxy file was not specified!\n" unless defined $proxy;
	die "Proxy file doesn't exists: $proxy\n" unless(-r $proxy);
	
	my $proxy_proxy  = $proxy . '.proxy';
	my $proxy_user   = $proxy . '.user';
	my $proxy_key    = $proxy . '.key';

	my $p = new FileHandle($proxy, 'r') 
		or die "Could not read to $proxy\n";
	
	umask(0077); # created file will be 0600
	my $pp = new FileHandle($proxy_proxy, 'w') 
		or die "Could not write to $proxy_proxy\n";
	my $pu = new FileHandle($proxy_user, 'w') 
		or die "Could not write to $proxy_proxy\n";
	my $pk = new FileHandle($proxy_key, 'w') 
		or die "Could not write to $proxy_proxy\n";

	# states:
	#    0 -- before proxy certificate (begining of file)
	#    1 -- proxy certificate
	#    2 -- before proxy key
	#    3 -- proxy key
	#    4 -- before user certificate
	#    5 -- user certificate
	my ($line, $state) = ('', 0);
	while(defined ($line = $p->getline())) {
		# proxy certificate
		if($state == 0 and $line =~ /^-----BEGIN CERTIFICATE-----/) {
			$pp->print($line);
			$state = 1;
		}
		elsif($state == 1 and $line =~ /^-----END CERTIFICATE-----/) {
			$pp->print($line);
			$pp->close();
			$state = 2;
		}
		elsif($state == 1) {
			$pp->print($line);
		}
		elsif($state == 2 and $line =~ /^-----BEGIN RSA PRIVATE KEY-----/) {
			$pk->print($line);
			$state = 3;
		}
		elsif($state == 3 and $line =~ /^-----END RSA PRIVATE KEY-----/) {
			$pk->print($line);
			$pk->close();
			$state = 4;
		}
		elsif($state == 3) {
			$pk->print($line);
		}
		# user certificate
		elsif($state == 4 and $line =~ /^-----BEGIN CERTIFICATE-----/) {
			$pu->print($line);
			$state = 5;
		}
		elsif($state == 5 and $line =~ /^-----END CERTIFICATE-----/) {
			$pu->print($line);
			$pu->close();
			$state = 6;
		}
		elsif($state == 5) {
			$pu->print($line);
		}
			
	}

	$ENV{HTTPS_CERT_FILE} = $proxy_proxy;
	$ENV{HTTPS_KEY_FILE} = $proxy_key;
	$ENV{HTTPS_CA_FILE} = $proxy_user;

	$isSplit = 1;
}

# cleans up the split pieces
sub cleanSplitProxy {
	# do not work if there is no work to do
	return unless $isSplit;
	
	die "Proxy file was not specified!\n" unless defined $proxy;
	die "Proxy file doesn't exists: $proxy\n" unless(-r $proxy);

	for my $p ('.proxy', '.user', '.key') {
		unlink $proxy . $p or warn "Could not remove $proxy.$p\n";
	}
	$isSplit = 0;
}

END {
	cleanSplitProxy();
}
	
1;


